/**
 * Copyright (c) 2009-2012 EBM WebSourcing, 2012-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.bc.sftp.authentication;

import java.io.IOException;

import org.ow2.petals.bc.sftp.authentication.exception.AuthenticationException;
import org.ow2.petals.bc.sftp.authentication.exception.UserMissingException;

import com.ebmwebsourcing.easycommons.lang.StringHelper;
import com.sshtools.j2ssh.authentication.SshAuthenticationClient;

/**
 * Root of a collection of utility class used to hold the informations used by
 * J2SSH to perform an authentication onto the SSH server.
 * 
 * @author Charles Casadei - EBM WebSourcing
 */
public abstract class SshAuthentication {

    /**
     * The username on the Ssh server.
     */
    private final String user;

    /**
     * Default constructor. Initialize an instance of SshAuthentication with the
     * username used for authenticate the component onto the SSH server.
     * 
     * @param user
     *            the username used for SSH authentication
     * @throws AuthenticationException
     */
    protected SshAuthentication(final String user) throws AuthenticationException {
        // Assign the given user to the current instance's user
        this.user = user;

        // Validates the configuration
        this.validate();
    }

    /**
     * Accessor the the username used for SSH authentication.
     * 
     * @return the username used for SSH authentication
     */
    public final String getUser() {
        return this.user;
    }

    /**
     * Builds the object used by the SFTPClientPool to perform the
     * authentication on the SSH server.
     * 
     * @return an object used by the SFTPClientPool to perform the
     *         authentication
     * @throws SshAuthenticationException
     */
    public abstract SshAuthenticationClient getAuthenticationClient() throws IOException;

    /**
     * Returns the key used to store an instance of this class (or any of its
     * subclass) into the SftpClientPool.
     * 
     * @return the key used to retrieve the ssh client in the SftpClientPool
     */
    public abstract String getKey();

    /**
     * Return the object content as String (Overrides the toString method).
     * 
     * @return the object content as String
     */
    @Override
    public String toString() {
        return "(user=" + this.user + " ; ";
    }

    /**
     * Checks if the given informations are OK.
     * 
     * @throws AuthenticationException
     *             if the configuration is not valid
     */
    private final void validate() throws AuthenticationException {
        if (StringHelper.isNullOrEmpty(this.user)) {
            throw new UserMissingException();
        }
    }
}
